import sys
import collections
from optparse import OptionParser,OptionGroup

#Author: Martin Kapun
#########################################################   HELP   #########################################################################
usage="usage: %prog --cand candidates.txt --full SNPs.txt --distance 100000 --output out"
parser = OptionParser()
group=OptionGroup(parser,"""

H E L P :
_________

Description:
This script allows to pick non-candidate SNPs which are located in a user defined distance (--distance) upstreams of a candidate SNPs and outputs the corresponding line of the full SNP file (--full). Note, if the distance between of a candidate to position zero is smaller than the user defined distance, the program will search downstreams of the candidate for a non-candidate SNP.

Input:
--cand and --full:

Any text file with at least two columns corresponding to Chromosome and position.

--distance
upstream distance in base pairs.
	""") 

#########################################################   CODE   #########################################################################


parser.add_option("-c", "--cand", dest="c", help="candidates cmh file")
parser.add_option("-f", "--full", dest="f", help="full SNP cmh file")
parser.add_option("-o", "--output", dest="o", help="outfile")
parser.add_option("-d", "--distance", dest="d", help="minimum distance of random SNP to candidate SNP")

parser.add_option_group(group)
(options, args) = parser.parse_args()

def positionhash(x):
	poshash=collections.defaultdict(lambda:collections.defaultdict(lambda:""))
	for l in open(x,"r"):
		if "position" not in l:
			a=l.split()
			pos=int(a[1])
			chrom=a[0]
			poshash[chrom][pos]=l
	return poshash

cand=positionhash(options.c)
full=positionhash(options.f)
distance=int(options.d)	 
out=open(options.o,"w")
count=0

for chrom,value in cand.items():
	for pos in value.keys():
		distance=int(options.d)	
		count+=1
		if pos-distance<=0:
			while pos+distance not in full[chrom]:
				distance+=1

			out.write(full[chrom][pos+distance])
		else:
			while pos-distance not in full[chrom]:
				distance-=1

			out.write(full[chrom][pos-distance])
		if count%100==0:
			print count,"SNPs processed"