import sys
from optparse import OptionParser,OptionGroup
import collections 

#Author: Martin Kapun
#########################################################   HELP   #########################################################################
usage="python %prog --input1 candidates.genes --input2 full.genes --output distribution_summary --g no --f no"
parser = OptionParser(usage=usage)
group=OptionGroup(parser,
"""
H E L P :
_________

Description:

This script compares two SNPeff outputs and produces summary tables of distribution of SNPs 1) on chromosmal arms, 2) in SNP features (based on the annotation with SNPeff 2.0.3), 3) inside and outside In(3R)Payne and 4) on the X versus the autosmomes. --input1 should be the candidate datatset and --input2 the complete SNP inventory. Note that in table 1 and 2, candidate SNPs will be substracted from the total SNP set.
Optionally the SNP produces lists of all FBGN numbers for the candidate dataset (--genelist1) and the total SNPset (--genelist2)

Input:

--input1 and --input2:

SNP sets annotated with SNPeff 2.0.3

Output: 
File with the extension .summary containing the following summary tables
Table1: SNP distribution in features based on SNPeff 2.0.3 
Table2: Chromosomal distribution 
Table3: SNP distribution inside and outside of In(3R)Payne
Table4: SNP distribution on X and autosomes

optionally two gene lists with the extensions _gl1 and _gl2 are created e.g. as an input for GoMiner
""")
	
	
#########################################################   CODE   #########################################################################

parser.add_option("-i", "--input1", dest="i", help="input file 1: candidate SNPs as an output from the link_snpeff_203.py program")
parser.add_option("-j", "--input2", dest="j", help="input file 2: total SNP set as an output from the link_snpeff_203.py program")
parser.add_option("-g", "--genelist2", dest="g", help="prints genelist for input2; put \"yes\", or \"no\"")
parser.add_option("-f", "--genelist1", dest="f", help="prints genelist for input1; put \"yes\", or \"no\"")
parser.add_option("-o", "--output", dest="p", help="define general output file")
parser.add_option_group(group)
(options, args) = parser.parse_args()


### inversion 3R(Payne) based on 
#proximal breakpoint: somewhere around 12401910
#distal breakpoint: somewhere around 20580518
def dict(inp1,inp2):
	dict=collections.defaultdict(lambda:0)
	for l in open(inp1,"r"):
		if "Chromosome" not in l.split()[0] and "na" not in l.split()[0] and l.rstrip()!="": 
			a=l.split("\t")
			dict[a[-4]]
	for l in open(inp2,"r"):
		if "Chromosome" not in l.split()[0] and "na" not in l.split()[0] and l.rstrip()!="": 
			a=l.split("\t")
			dict[a[-4]]
	return dict				
		


def summary(data):
	genes=open(data,"r")
	payne_prox=int(12401910)
	payne_dist=int(20580518)
	chromosomes=["2L","2LHet","2R","2RHet","3L","3LHet","3R","3Rinv","3Rout","3RHet","4","X","Xhet","Yhet","Aut","total"]
	genelist,posdict={},{}
	cdict=collections.defaultdict(lambda:0)
	fdict={"UTR_3_PRIME":0,"UTR_5_PRIME":0,"DOWNSTREAM":0,"INTERGENIC":0,"INTRON":0,"NON_SYNONYMOUS_CODING":0,"SYNONYMOUS_CODING":0,"UPSTREAM":0,"other":0,"total":0}
	for l in genes:
		if  l.split()[0] != "Chromosome" and l.rstrip()!="":
			posdict[l.split()[0]+"_"+l.split()[1]]=1
			fdict["total"]+=1
			chrom=l.split("\t")[0]
			pos=l.split("\t")[1]
			#print chrom, pos
			effect=l.split("\t")[-4:-3]
			gene_name=l.split("\t")[-5:-4]
			gene_id=l.split("\t")[-6:-5]
			if "na" not in gene_name:
				genelist[gene_name[0]]=gene_id[0]
			if effect[0] in fdict:
				fdict[effect[0]]+=1
			else:
				fdict["other"]+=1
	print (len(posdict))
	for k in posdict.keys():
		#print k
		chrom=k.split("_")[0]
		pos=k.split("_")[1]
		cdict["total"]+=1
		cdict[chrom]+=1
		if chrom=="3R" and int(pos)>payne_prox and int(pos)<payne_dist:
			cdict["3Rinv"]+=1
		if chrom=="3R" and int(pos)<payne_prox:
			cdict["3Rout"]+=1
		if chrom=="3R" and int(pos)>payne_dist:
			cdict["3Rout"]+=1
		if chrom!="X":
			cdict["Aut"]+=1		
			
	return fdict,cdict,genelist,posdict

def names(inp):
	if "/" in inp:
		name=inp.split("/")[-1]
	else:
		name=inp
	return name

fdict_inpj,cdict_inpj,genelist_inpj,pos_inpj=summary(str(options.j))
fdict_inpi,cdict_inpi,genelist_inpi,pos_inpi=summary(str(options.i))

outg=open(str(options.p)+".summary","w")

names_i=names(str(options.i))
names_j=names(str(options.j))
#### print summary of SNP effects ######
outg.write("Snp_Effect\tcount:"+names_j+"\tfreq:"+names_j+"\tcount:"+names_i+"\tfreq:"+names_i+"\n")
outg.write("__________________________________________________________________________________________"+"\n")
for effect,v in sorted(fdict_inpj.items()):
	if effect!="total":
		outg.write(effect+"\t"+str(v-fdict_inpi[effect])+"\t"+str(float(v-fdict_inpi[effect])/float(fdict_inpj["total"]-fdict_inpi["total"]))+"\t"+str(fdict_inpi[effect])+"\t"+str(float(fdict_inpi[effect])/float(fdict_inpi["total"]))+"\n")
outg.write("__________________________________________________________________________________________"+"\n")
outg.write("total\t"+str(fdict_inpj["total"]-fdict_inpi["total"])+"\t1.0\t"+str(fdict_inpi["total"])+"\t"+str(float(fdict_inpi["total"])/float(fdict_inpi["total"]))+"\n")
outg.write("\n")
#### print summary of distribution across genome ######
outg.write("Chromosome\tcount:"+names_j+"\tfreq:"+names_j+"\tcount:"+names_i+"\tfreq:"+names_i+"\n")
outg.write("__________________________________________________________________________________________"+"\n")
for chromosome,v in sorted(cdict_inpj.items()):
	if chromosome!="total" and chromosome!="3Rinv" and chromosome!="3Rout"and chromosome!="Aut":
		outg.write(chromosome+"\t"+str(v-cdict_inpi[chromosome])+"\t"+str(float(v-cdict_inpi[chromosome])/float(cdict_inpj["total"]-cdict_inpi["total"]))+"\t"+str(cdict_inpi[chromosome])+"\t"+str(float(cdict_inpi[chromosome])/float(cdict_inpi["total"]))+"\n")
outg.write("__________________________________________________________________________________________"+"\n")
outg.write("total\t"+str(cdict_inpj["total"]-cdict_inpi["total"])+"\t1.0\t"+str(cdict_inpi["total"])+"\t"+str(float(cdict_inpi["total"])/float(cdict_inpi["total"]))+"\n")
outg.write("\n")
outg.write("""SNPs_inside_or_outside_of_In(3R)Payne:"""+"\n")
outg.write("location\tcount:"+names_j+"\tfreq:"+names_j+"\tcount:"+names_i+"\tfreq:"+names_i+"\n")
outg.write("__________________________________________________________________________________________"+"\n")
outg.write("3Rinv\t"+str(cdict_inpj["3Rinv"]-cdict_inpi["3Rinv"])+"\t"+str(float(cdict_inpj["3Rinv"]-cdict_inpi["3Rinv"])/float(cdict_inpj["3R"]-cdict_inpi["3R"]))+"\t"+str(cdict_inpi["3Rinv"])+"\t"+str(float(cdict_inpi["3Rinv"])/float(cdict_inpi["3R"]))+"\n")
outg.write("3Rout\t"+str(cdict_inpj["3Rout"]-cdict_inpi["3Rout"])+"\t"+str(float(cdict_inpj["3Rout"]-cdict_inpi["3Rout"])/float(cdict_inpj["3R"]-cdict_inpi["3R"]))+"\t"+str(cdict_inpi["3Rout"])+"\t"+str(float(cdict_inpi["3Rout"])/float(cdict_inpi["3R"]))+"\n")
outg.write("__________________________________________________________________________________________"+"\n")
outg.write("3R\t"+str(cdict_inpj["3R"]-cdict_inpi["3R"])+"\t1.0\t"+str(cdict_inpi["3R"])+"\t"+str(float(cdict_inpi["3R"])/float(cdict_inpi["3R"]))+"\n")
outg.write("\n")
outg.write("""Ratio X to autosome:"""+"\n")
outg.write("location\tcount:"+names_j+"\tfreq:"+names_j+"\tcount:"+names_i+"\tfreq:"+names_i+"\n")
outg.write("__________________________________________________________________________________________"+"\n")
outg.write("X\t"+str(cdict_inpj["X"]-cdict_inpi["X"])+"\t"+str(float(cdict_inpj["X"]-cdict_inpi["X"])/float(cdict_inpj["total"]-cdict_inpi["total"]))+"\t"+str(cdict_inpi["X"])+"\t"+str(float(cdict_inpi["X"])/float(cdict_inpi["total"]))+"\n")
outg.write("Aut\t"+str(cdict_inpj["Aut"]-cdict_inpi["Aut"])+"\t"+str(float(cdict_inpj["Aut"]-cdict_inpi["Aut"])/float(cdict_inpj["total"]-cdict_inpi["total"]))+"\t"+str(cdict_inpi["Aut"])+"\t"+str(float(cdict_inpi["Aut"])/float(cdict_inpi["total"]))+"\n")
outg.write("__________________________________________________________________________________________"+"\n")
outg.write("total\t"+str(cdict_inpj["total"]-cdict_inpi["total"])+"\t"+str(float(cdict_inpj["total"]-cdict_inpi["total"])/float(cdict_inpj["total"]-cdict_inpi["total"]))+"\t"+str(cdict_inpi["total"])+"\t"+str(float(cdict_inpi["total"])/float(cdict_inpi["total"]))+"\n")

if str(options.g)=="yes":
	outgl1=open(str(options.p)+"_gl2.txt","w")
	#outgl1.write( "#SNPs overlap with "+str(len(genelist_inpj))+" genes in "+str(options.j)+"\n")
	for k,v in sorted(genelist_inpj.items()):
		if v!="":
			outgl1.write(v+"\n")
if str(options.f)=="yes":
	outgl2=open(str(options.p)+"_gl1.txt","w")
	#outgl2.write( "#SNPs overlap with "+str(len(genelist_inpi))+" genes in "+str(options.i)+"\n")
	for k,v in sorted(genelist_inpi.items()):
		if v!="":
			outgl2.write(v+"\n")